from fastapi import APIRouter
from typing import Dict
from src.modules.IndustrialAutomation import service_analog as analogService
from src.modules.IndustrialAutomation import service_digital as digitalService
from src.modules.universal_models.value import FloatValueModel, BoolValueModel

router = APIRouter(prefix="/industrialAutomation", tags=["Modules --> Industrial Automation Hat"])


#analog
analog = APIRouter(prefix="/analog")

#0-10 V output
@analog.get("/output/0-10V/all", description="Read all setted 0-10V outputs value", response_model=Dict[int, Dict[int, float]])
def get_0_10V_out_all():
    return analogService.read_0_10_out_all()

@analog.get("/output/0-10V/{stack}", description="Read setted 0-10V output values on specified stack", response_model=Dict[int, float])
def get_0_10V_out_stack(stack: int):
    return analogService.read_0_10_out_stack(stack=stack)

@analog.get("/output/0-10V/{stack}/{channel}", description="Read set value from selected 0-10V output", response_model=FloatValueModel)
def get_0_10V_out(stack: int, channel: int):
    return FloatValueModel(value=analogService.read_0_10_out(stack=stack, channel=channel))


@analog.post("/output/0-10V/all", description="Set output value to all 0-10V outputs", response_model=Dict[int, Dict[int, float]])
def post_0_10V_out_all(output: FloatValueModel):
    analogService.set_0_10_out_all(value=output.value)
    return analogService.read_0_10_out_all()

@analog.post("/output/0-10V/{stack}", description="Set output value to all 0-10V output on specified card", response_model=Dict[int, float])
def post_0_10V_out_stack(stack: int, output: FloatValueModel):
    analogService.set_0_10_out_stack(stack=stack, value=output.value)
    return analogService.read_0_10_out_stack(stack=stack)

@analog.post("/output/0-10V/{stack}/{channel}", description="Set output value to specified 0-10V output", response_model=FloatValueModel)
def post_0_10V_out(stack: int, channel: int, output: FloatValueModel):
    analogService.set_0_10_out(stack=stack, channel=channel, value=output.value)
    return FloatValueModel(value=analogService.read_0_10_out(stack=stack, channel=channel))


#0-10V input
@analog.get("/input/0-10V/all", description="Read all 0-10V inputs value", response_model=Dict[int, Dict[int, float]])
def get_0_10V_in_all():
    return analogService.read_0_10_in_all()

@analog.get("/input/0-10V/{stack}", description="Read all 0-10V inputs on specified card", response_model=Dict[int, float])
def get_0_10V_in_stack(stack: int):
    return analogService.read_0_10_in_stack(stack=stack)

@analog.get("/input/0-10V/{stack}/{channel}", description="Read specified 0-10V input", response_model=FloatValueModel)
def get_0_10V_in(stack: int, channel: int):
    return FloatValueModel(value=analogService.read_0_10_in(stack=stack, channel=channel))


#4-20mA output
@analog.get("/output/4-20mA/all", description="Read all setted 4-20mA output values", response_model=Dict[int, Dict[int, float]])
def get_4_20mA_out_all():
    return analogService.read_4_20_out_all()

@analog.get("/output/4-20mA/{stack}", description="Read all setted 4-20mA output values of specified card", response_model=Dict[int, float])
def get_4_20mA_out_stack(stack: int):
    return analogService.read_4_20_out_stack(stack=stack)

@analog.get("/output/4-20mA/{stack}/{channel}", description="Read setted 4-20mA output of specified channel", response_model=FloatValueModel)
def get_4_20mA_out(stack: int, channel: int):
    return analogService.read_4_20_out(stack=stack, channel=channel)


@analog.post("/output/4-20mA/all", description="Set output value to all 4-20mA outputs", response_model=Dict[int, Dict[int, float]])
def post_4_20mA_out_all(output: FloatValueModel):
    analogService.set_4_20_out_all(value=output.value)
    return analogService.read_4_20_out_all()

@analog.post("/output/4-20mA/{stack}", description="Set output value of 4-20mA outputs on specified card", response_model=Dict[int, float])
def post_4_20mA_out_stack(stack: int, output: FloatValueModel):
    analogService.set_4_20_out_stack(stack=stack, value=output.value)
    return analogService.read_4_20_out_stack(stack=stack)

@analog.post("/output/4-20mA/{stack}/{channel}", description="Set output value of specified 4-20mA output", response_model=FloatValueModel)
def post_4_20mA_out(stack: int, channel: int, output: FloatValueModel):
    analogService.set_4_20_out(stack=stack, channel=channel, value=output.value)
    return FloatValueModel(value=analogService.read_4_20_out(stack=stack, channel=channel))


#4-20mA input
@analog.get("/input/4-20mA/all", description="Read all 4-20mA input values", response_model=Dict[int, Dict[int, float]])
def get_4_20mA_out_all():
    return analogService.read_4_20_in_all()

@analog.get("/input/4-20mA/{stack}", description="Read all 4-20mA input values of specified card", response_model=Dict[int, float])
def get_4_20mA_out_stack(stack: int):
    return analogService.read_4_20_in_stack(stack=stack)

@analog.get("/input/4-20mA/{stack}/{channel}", description="Read 4-20mA input of specified channel", response_model=FloatValueModel)
def get_4_20mA_out(stack: int, channel: int):
    return analogService.read_4_20_in(stack=stack, channel=channel)







#digital
digital = APIRouter(prefix="/digital")

#opto inputs
@digital.get("/input/opto/all", description="Read all opto isolated inputs", response_model=Dict[int, Dict[int, bool]])
def get_opto_in_all():
    return digitalService.read_opto_all()

@digital.get("/input/opto/{stack}", description="Read all opto isolated inputs on specified card", response_model=Dict[int, bool])
def get_opto_in_stack(stack: int):
    return digitalService.read_opto_stack(stack=stack)

@digital.get("/input/opto/{stack}/{channel}", description="Read selected opto isolated input", response_model=BoolValueModel)
def get_opto_in(stack: int, channel:int):
    return BoolValueModel(value=digitalService.read_opto(stack=stack, channel=channel))


#open Drain
@digital.get("/output/OD/all", description="Read open drain PWM freq in percentage from all outputs", response_model=Dict[int, Dict[int, float]])
def get_od_all():
    return digitalService.read_od_pwm_all()

@digital.get("/output/OD/{stack}", description="Read open dran PWM freq in percentage tfrom all outputs on selected card", response_model=Dict[int, float])
def get_od_stack(stack: int):
    return digitalService.read_od_pwm_stack(stack=stack)

@digital.get("/output/OD/{stack}/{channel}", description="Read open drain freq in percentage from selected output", response_model=FloatValueModel)
def get_od(stack: int, channel: int):
    return FloatValueModel(value=digitalService.read_od_pwm(stack=stack, channel=channel))


@digital.post("/output/OD/all", description="Set open drain PWM freq in percentage to all outputs", response_model=Dict[int, Dict[int, float]])
def post_od_all(output: FloatValueModel):
    digitalService.set_od_pwm_all(value=output.value)
    return digitalService.read_od_pwm_all()

@digital.post("/output/OD/{stack}", description="Set open drain freq in percentage to all outputs on selected card", response_model=Dict[int, float])
def post_od_stack(stack: int, output: FloatValueModel):
    digitalService.set_od_pwm_stack(stack=stack, value=output.value)
    return digitalService.read_od_pwm_stack(stack=stack)

@digital.post("/output/OD/{stack}/{channel}", description="Set open drain freq in percentage to selected channel", response_model=FloatValueModel)
def post_od(stack: int, channel: int, output: FloatValueModel):
    digitalService.set_od_pwm(stack=stack, channel=channel, value=output.value)
    return FloatValueModel(value=digitalService.read_od_pwm(stack=stack, channel=channel))










router.include_router(router=analog)
router.include_router(router=digital)