import os, megaind


stacks = [int(h) for h in os.getenv("MODULE_INDUSTRIALAUTOMATION_LEVELS", "")]
if not stacks:
    print("MODULE_INDUSTRIALAUTOMATION_LEVELS is not set or invalid")

for i in stacks:
    print(f"Configured IndustrialAutomation card: {i}")



def read_0_10_out(stack: int, channel: int) -> float:
    """
    Read set value in selected 0-10V output
    """
    if stack in stacks and 1 <= channel <= 4:
        try:
            return megaind.get0_10Out(stack=stack, channel=channel)
        except:
            return -512
    return -512

def read_0_10_out_stack(stack: int) -> dict[int, float]:
    """
    Read set values in 0-10V output on selected card
    """
    return {channel: read_0_10_out(stack=stack, channel=channel) for channel in range(1, 5)}

def read_0_10_out_all() -> dict[int, dict[int, float]]:
    """
    Read set values in 0-10V outputs on all cards
    """
    return {stack: read_0_10_out_stack(stack=stack) for stack in stacks}


def set_0_10_out(stack: int, channel: int, value: float):
    """
    Set output voltage of 0-10V output to specified output
    """
    if stack in stacks and 1 <= channel <= 4:
        if value > 10:
            value = 10
        elif value < 0:
            value = 0
        
        try:
            megaind.set0_10Out(stack=stack, channel=channel, value=value)
        except:
            print("error set 0-10V output value")

def set_0_10_out_stack(stack: int, value: float):
    """
    Set output voltage of 0-10V output on specified card
    """
    for channel in range(1, 5):
        set_0_10_out(stack=stack, channel=channel, value=value)

def set_0_10_out_all(value: float):
    """
    Set output voltage of 0-10V output on all card
    """
    for stack in stacks:
        set_0_10_out_stack(stack=stack, value=value)



def read_0_10_in(stack: int, channel: int) -> float:
    """
    Read value in selected 0-10V input
    """
    if stack in stacks and 1 <= channel <= 4:
        try:
            return megaind.get0_10In(stack=stack, channel=channel)
        except:
            return -512
    return -512

def read_0_10_in_stack(stack: int) -> dict[int, float]:
    """
    Read values in 0-10V input on selected card
    """
    return {channel: read_0_10_in(stack=stack, channel=channel) for channel in range(1, 5)}

def read_0_10_in_all() -> dict[int, dict[int, float]]:
    """
    Read values in 0-10V inputs on all cards
    """
    return {stack: read_0_10_in_stack(stack=stack) for stack in stacks}



def read_4_20_in(stack: int, channel: int) -> float:
    """
    Read values from specified 4-20mA input
    """
    if stack in stacks and 1 <= channel <= 4:
        try:
            return megaind.get4_20In(stack=stack, channel=channel)
        except:
            return -512
    return -512

def read_4_20_in_stack(stack: int) -> dict[int, float]:
    """
    Read values from all 4-20mA inputs on specified card
    """
    return {channel: read_4_20_in(stack=stack, channel=channel) for channel in range(1, 5)}

def read_4_20_in_all() -> dict[int, dict[int, float]]:
    """
    Read values from all 4-20mA inputs
    """
    return {stack: read_4_20_in_stack(stack=stack) for stack in stacks}



def read_4_20_out(stack: int, channel: int) -> float:
    """
    Read setted value from specified 4-20mA output
    """
    if stack in stacks and 1 <= channel <= 4:
        try:
            return megaind.get4_20Out(stack=stack, channel=channel)
        except:
            return -512
    return -512

def read_4_20_out_stack(stack: int) -> dict[int, float]:
    """
    Read setted values from all 4-20mA outputs on specified card
    """
    return {channel: read_4_20_out(stack=stack, channel=channel) for channel in range(1, 5)}

def read_4_20_out_all() -> dict[int, dict[int, float]]:
    """
    Read setted values from all 4-20mA outputs
    """
    return {stack: read_4_20_out_stack(stack=stack) for stack in stacks}


def set_4_20_out(stack: int, channel: int, value: float):
    """
    Set output value for specified 4-20mA output
    """
    if stack in stacks and 1<= channel <= 4:
        if value > 20:
            value = 20
        elif value < 4:
            value = 4
        
        try:
            megaind.set4_20Out(stack=stack, channel=channel, value=value)
        except:
            print("Error set 4-20mA output")

def set_4_20_out_stack(stack: int, value: float):
    """
    Set output value to all 4-20mA on specified card
    """
    for channel in range(1, 5):
        set_4_20_out(stack=stack, channel=channel, value=value)

def set_4_20_out_all(value: float):
    """
    Set output value to all for 4-20mA outputs
    """
    for stack in stacks:
        set_4_20_out_stack(stack=stack, value=value)

