import os, megaind



stacks = [int(h) for h in os.getenv("MODULE_INDUSTRIALAUTOMATION_LEVELS", "")]
if not stacks:
    print("MODULE_INDUSTRIALAUTOMATION_LEVELS is not set or invalid")

for i in stacks:
    print(f"Configured IndustrialAutomation card: {i}")



def read_opto(stack: int, opto: int):
    """
    Read specified opto input state
    """
    if stack in stacks and 1 <= opto <= 4:
        try:
            if megaind.getOptoCh(stack=stack, channel=opto) == 1:
                return True
            else:
                return False
        except:
            return False
    return False

def read_opto_stack(stack: int):
    """
    Read all opto inputs state on specified stack/card 
    """
    return {opto: read_opto(stack=stack, opto=opto) for opto in range(1, 5)}

def read_opto_all():
    """
    Read all opto inputs state
    """
    return {stack: read_opto_stack(stack=stack) for stack in stacks}



def set_od_pwm(stack: int, channel: int, value: float):
    """
    Set specified OpenDrain PWM percentage
    """
    if stack in stacks and 1 <= channel <= 4:
        if value < 0:
            value = 0
        elif value > 100:
            value = 100

        megaind.setOdPWM(stack=stack, channel=channel, value=value)

def set_od_pwm_stack(stack: int, value: bool):
    """
    Set all OpenDrain PWM percentage on specified stack/card
    """
    (set_od_pwm(stack=stack, channel=ch, value=value) for ch in range(1, 5))

def set_od_pwm_all(value: bool):
    """
    Set all OpenDrain PWM percentage to requested state
    """
    (set_od_pwm_stack(stack=stack, value=value) for stack in stacks)


def read_od_pwm(stack: int, channel: int) -> float:
    """
    Read specified OpenDrain PWM percentage
    """
    if stack in stacks and 1 <= channel <= 4:
        try:
            return megaind.getOdPWM(stack=stack, channel=channel)
        except:
            return -512
    return -512

def read_od_pwm_stack(stack: int) -> dict[int, float]:
    """
    Read all OpenDrain PWM percentage on specified stack/card
    """
    return {channel: read_od_pwm(stack=stack, channel=channel) for channel in range(1, 5)}

def read_od_pwm_all() -> dict[int, dict[int, float]]:
    """
    Read all OpenDrain PWM percentage
    """
    return {stack: read_od_pwm_stack(stack=stack) for stack in stacks}









"""
TODO
OptoCount + Opto frequency
LED control
RTC
Owire Bus
"""