import src.modules.RTD8.service as service
from fastapi import APIRouter
from typing import Dict
from src.modules.universal_models.value import FloatValueModel



router = APIRouter(prefix="/RTD8", tags=["Modules --> RTD8 card"])



@router.get("/temp/all", summary="Read all RTD8 cards values in Celsius", response_model=Dict[int, Dict[int, float]])
def get_all_celsius():
    return service.read_temp_all()

@router.get("/temp/{stack}", summary="Read specified RTD8 card values in Celsius", response_model=Dict[int, float])
def get_stack_celsius(stack: int):
    return service.read_temp_stack(stack)

@router.get("/temp/{stack}/{channel}", summary="Read specified RTD8 card value of specified channel in Celsius", response_model=float)
def get_channel_celsius(stack: int, channel: int):
    return FloatValueModel(service=service.read_temp(stack, channel))



@router.get("/res/all", summary="Read all RTD8 cards values in Ohm", response_model=Dict[int, Dict[int, float]])
def get_all_celsius():
    return service.read_resistance_all()

@router.get("/res/{stack}", summary="Read specified RTD8 card values in Ohm", response_model=Dict[int, float])
def get_stack_celsius(stack: int):
    return service.read_resistance_stack(stack)

@router.get("/res/{stack}/{channel}", summary="Read specified RTD8 card value of specified channel in Ohm", response_model=FloatValueModel)
def get_channel_celsius(stack: int, channel: int):
    return FloatValueModel(value=service.read_resistance(stack, channel))