from fastapi import APIRouter
import src.modules.Rel4HVI4.service as service
from typing import Dict
from src.modules.universal_models.value import BoolValueModel



router = APIRouter(prefix="/rel4hvi4", tags=["Modules --> Rel4HVI4"])



@router.post("/relay/all", description="Set all relays to required state (1/0)", response_model=Dict[int, Dict[int, bool]])
def post_relay_all(state: BoolValueModel):
    service.set_relay_all(value=state.value)
    return service.read_relay_all()

@router.post("/relay/{stack}", description="Set all relays on specified card to required state (1/0)", response_model=Dict[int, float])
def post_relay_stack(stack: int, state: BoolValueModel):
    service.set_relay_stack(stack=stack, value=state.value)
    return service.read_opto_stack(stack=stack)

@router.post("/relay/{stack}/{channel}", description="Set all relays on specified card to required state (1/0)", response_model=BoolValueModel)
def post_relay_stack(stack: int, channel: int, state: BoolValueModel):
    service.set_relay(stack=stack, relay=channel, value=state.value)
    return BoolValueModel(value=service.read_relay(stack=stack, relay=channel))


@router.get("/relay/all", description="Get state of all relays", response_model=Dict[int, Dict[int, bool]])
def get_relay_all():
    return service.read_relay_all()

@router.get("/relay/{stack}", description="Get all relays state of specified card", response_model=Dict[int, bool])
def get_relay_stack(stack: int):
    return service.read_relay_stack(stack=stack)

@router.get("/relay/{stack}/{channel}", description="Get state of selected relay", response_model=BoolValueModel)
def get_relay(stack: int, channel: int):
    return BoolValueModel(value=service.read_relay(stack=stack, relay=channel))



@router.get("/opto/all", description="Get input state of all opto inputs", response_model=Dict[int, Dict[int, bool]])
def get_opto_all():
    return service.read_opto_all()

@router.get("/opto/{stack}", description="Get input state all opto inputs on selected card", response_model=Dict[int, bool])
def get_opto_stack(stack: int):
    return service.read_opto_stack(stack=stack)

@router.get("/opto/{stack}/{channel}", description="Get state of selected input", response_model=BoolValueModel)
def get_opto(stack: int, channel: int):
    return BoolValueModel(value=service.read_opto(stack=stack, opto=channel))