from fastapi import APIRouter, status, HTTPException
import src.variables.service as service
from src.variables.models import SetVariable


router = APIRouter(prefix="/variables", tags=["Variables system"])


@router.post("/{variable}", description="Set variable")
def set_variable(variable: str, params: SetVariable):
    service.set_variable(variable=variable, data=params.data, default=params.default)

@router.get("/{variable}", description="Get variable data", response_model=str)
def get_variable(variable: str):
    try:
        return str(service.get_variable(variable=variable))
    except:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND)

@router.get("/", description="Get variables list", response_model=list)
def get_variables():
    return service.variables.keys()

@router.delete("/{variable}", description="Remove variable")
def delete_variable(variable: str):
    service.remove_variable(variable=variable)