import os, re


variables: dict = {}


def __save_data(variable, data):
    with open(variable, "w") as file:
        file.write(str(data))
    

def __load_data(variable):
    with open(variable, "r") as file:
        return file.read()

def __create_file_path(variable: str):
    default_path: str = os.getenv("VARIABLES_SAVE_DIR", "variables_storage")
    if not os.path.exists(default_path):
        os.makedirs(default_path)
    return "{}/{}".format(default_path, re.sub(r"[^\w\-_.]", "_", variable))


def set_variable(variable: str, data, default: bool):
    """
    Set variable
    """
    variables[variable] = data

    if default:
        __save_data(variable=__create_file_path(variable), data=data)


def load_variable(variable: str) -> str:
    """
    Load variable from File if exist
    """
    try:
        return __load_data(variable=__create_file_path(variable=variable))
    except FileNotFoundError:
        raise FileNotFoundError(f"Unnable find variable: {variable}")
    except:
        raise ValueError((f"Error with loading variable: {variable}"))
    

def get_variable(variable: str) -> str | None:
    """
    Get variable if exist
    """
    if variable in variables:
        return variables[variable]
    else:
        return None


def get_variables() -> list[str]:
    return variables.keys()
    

def remove_variable(variable: str):
    if variable in variables:
        variables.pop(variable)
    
    filePath = __create_file_path(variable)
    if os.path.exists(filePath):
        os.remove(filePath)


#autoload variables from files
__create_file_path("none")
for file in os.listdir(os.getenv("VARIABLES_SAVE_DIR", "variables_storage")):
    variables[file] = load_variable(file)